using GLib;
using Gtk;
using Hildon;

public class Playlist : PageWidget
{	
	private XMList playlists;
	private XMList itemlist;
	Item[] itemslist;
	int x;
	int y;
	Scroller scroller;
	private bool needs_update;
	private bool reordering;
	private Item[] swapped;
	private int download_index;
	private string downloaded_filename;
	
	construct
	{
		downloaded_filename = "";
		needs_update = true;
		download_index = 0;
	}
	
	public override void add_signals()
	{
		get_toolbar_button(0).clicked += () => {
			this.get_knots().get_browser().reveal();
		};
		get_toolbar_button(1).clicked += () => {
			this.get_knots().get_player().play(this.get_selected_playlist_id().to_string(), this.get_knots().get_player().PLAYLIST);
		};
		get_toolbar_button(2).clicked += () => {
			reorder();
		};
		get_toolbar_button(3).clicked += () => {
			new_playlist();
		};
		get_toolbar_button(4).clicked += () => {
			delete_playlist();
		};
	}
	
	public void reorder()
	{
		if (reordering)
		{
			for (int i = 0; i < swapped.length; i++)
			{
				swapped[i].lowlight();
			}
			swapped = {};
		}
		
		var children = scroller.get_container().get_children();
		foreach (Gtk.Widget child in children)
		{
			if (child is Item)
			{
				if (!reordering)
					((Item)child).button_release_event += swap_order;
				else
					((Item)child).button_release_event -= swap_order;
			}
		}
		for (int i = 0; i < 5; i++)
		{
			if (i != 2)
				get_toolbar_button(i).set_sensitive(reordering);
		}
		if (!reordering)
			get_knots().alert("Click on items to swap them");
		scroller.set_scroll_policy(false, reordering);
		get_playlist_widget().set_sensitive(reordering);
		scroller.enable_clicking(reordering);
		reordering = !reordering;
		
	}
	
	public override void orientation_change()
	{
		base.orientation_change();
		re_add_items();
	}
	
	public bool swap_order(Gtk.Widget w, Gdk.EventButton e)
	{
		Item item = (Item)w;
		swapped += item;
		item.highlight();
		if (swapped.length == 2)
		{
			if (swapped[0] != swapped[1])
			{
				get_knots().get_connection().set_url_cache(false);
				get_knots().get_connection().fetch("/external/swap_playlist_items?pid=" + swapped[0].get_item_attribute("pid") + "&with=" + swapped[1].get_item_attribute("pid"));
				Gtk.Allocation first = swapped[0].allocation;
				((Gtk.Fixed)scroller.get_container()).move(swapped[0], swapped[1].allocation.x, swapped[1].allocation.y);
				((Gtk.Fixed)scroller.get_container()).move(swapped[1], first.x, first.y);
			}
			for (int i = 0; i < swapped.length; i++)
			{
				swapped[i].lowlight();
			}
			swapped = {};
		}
		return false;
	}
	
	public void new_playlist()
	{
		MessageDialog dialog = new MessageDialog.with_markup(null, 0, Gtk.MessageType.QUESTION, Gtk.ButtonsType.OK_CANCEL, "Playlist name");
		((Label)dialog.label).selectable = false;
		gtk_window_set_portrait_flags(dialog, Hildon.PortraitFlags.SUPPORT);
		dialog.set_image(new Gtk.Image.from_file("/usr/share/knots/knots_button_playlist.png"));
		Hildon.Entry playlist_name = new Hildon.Entry(SizeType.FINGER_HEIGHT);
		dialog.vbox.pack_start(playlist_name, true, true, 1);
		playlist_name.show();
		var response = dialog.run();
		if (response == ResponseType.OK)
		{
			get_knots().get_connection().set_url_cache(false);
			string pid = get_knots().get_connection().fetch("/external/new_playlist?name=" + playlist_name.get_text());
			update_playlists();
			select_playlist(pid.to_int());
			update_items();
		}
		dialog.destroy();
	}
	
	public void delete_playlist()
	{
		MessageDialog dialog = new MessageDialog.with_markup(null, 0, Gtk.MessageType.QUESTION, Gtk.ButtonsType.OK_CANCEL, "Are you sure you want to delete this playlist?");
		((Label)dialog.label).selectable = false;
		gtk_window_set_portrait_flags(dialog, Hildon.PortraitFlags.SUPPORT);
		dialog.set_image(new Gtk.Image.from_file("/usr/share/knots/knots_button_playlist.png"));
		var response = dialog.run();
		if (response == ResponseType.OK)
		{
			clear_list();
			get_knots().get_connection().set_url_cache(false);
			get_knots().get_connection().fetch("/external/delete_playlist?id=" + get_selected_playlist_id().to_string());
			update_playlists();
			select_playlist(1);
			update_items();
		}
		dialog.destroy();
	}
	
	public override Gtk.Widget create_content()
	{
		Gtk.VBox vb = new Gtk.VBox(false, 0);
		Hildon.Button cb = new Hildon.Button.with_text (SizeType.FINGER_HEIGHT, ButtonArrangement.VERTICAL, "Current playlist", "");
		cb.clicked += change_playlist;
		vb.pack_start(cb, false, true, 2);
		Gtk.Fixed f = new Gtk.Fixed();
		scroller = new Scroller(f);
		vb.pack_start(scroller, true, true, 0);
		return vb;
	}
	
	public void change_playlist()
	{
		if (playlists.itemcount() > 1)
		{
			var dialog = new PickerDialog (get_knots().get_window());
			gtk_window_set_portrait_flags(dialog, Hildon.PortraitFlags.SUPPORT);
			dialog.set_title ("Select playlist");
			ListStore playlistnames = new ListStore (1, typeof (string));
			TreeIter iter;
			int index = 0;
			int selected = 0;
			foreach (HashTable<string,string> ht in playlists.items())
			{
				string playlist_name = (string)ht.lookup("name");
				playlistnames.append (out iter);
				playlistnames.set_value (iter, 0, playlist_name);
				if (playlist_name == get_playlist_widget().get_value())
				{
					selected = index;
				}
				index += 1;
			}
			var selector = new TouchSelector ();
			selector.append_text_column (playlistnames, true);
			dialog.set_selector (selector);
			selector.set_active(0, selected);
			var res = dialog.run ();
			if (res == ResponseType.OK)
			{
				HashTable<string,string> sp = playlists.items()[selector.get_active(0)];
				select_playlist((int)((string)sp.lookup("id")).to_int());
				update_items();
			}
			dialog.destroy ();
		}
	}
	
	public bool playlist_contains_item(int id)
	{
		if (itemlist != null)
		{
			foreach(HashTable<string,string> ht in itemlist.items())
			{
				if (((string)ht.lookup("id")).to_int() == id)
					return true;
			}
		}
		return false;
	}
	
	public Hildon.Button get_playlist_widget()
	{
		return (Hildon.Button)((Gtk.VBox)get_content()).get_children().nth_data(0);
	}
	
	public void update_playlists()
	{
		get_knots().get_connection().set_url_cache(false);
		playlists = new XMList(get_knots().get_connection().fetch("/external/playlists"));
	}
	
	public bool select_default_playlist()
	{
		select_playlist(1);
		return false;
	}
	
	public void select_playlist(int id)
	{
		int index = 0;
		foreach (HashTable<string,string> ht in playlists.items())
		{
			if (((string)ht.lookup("id")).to_int() == id)
			{
				get_playlist_widget().set_value((string)ht.lookup("name"));
				return;
			}
			index += 1;
		}
	}
	
	public int get_selected_playlist_id()
	{
		if (playlists != null)
		{
			foreach (HashTable<string,string> ht in playlists.items())
			{
				string name = (string)ht.lookup("name");
				if (name == get_playlist_widget().get_value())
					return ((string)ht.lookup("id")).to_int();	
			}
		}
		return -1;
	}
	
	public void update_list()
	{
		get_knots().get_connection().set_url_cache(false);
		itemlist = new XMList(get_knots().get_connection().fetch("/external/playlist?id=" + get_selected_playlist_id().to_string()));
	}
	
	public void update_items()
	{
		if (get_selected_playlist_id() != -1)
		{
			clear_list();
			gtk_window_set_progress_indicator(get_knots().get_window(), 1);
			try
			{
				Thread.create(fetch_list, true);
			}
			catch (ThreadError e)
			{
			
			}
		}
	}
	
	public void* fetch_list()
	{
		get_knots().get_connection().set_url_cache(false);
		itemlist = new XMList(get_knots().get_connection().fetch("/external/playlist?id=" + get_selected_playlist_id().to_string()));
		GLib.Idle.add(show_list);
		return null;
	}
	
	public bool show_list()
	{
		itemslist = new Item[0];
		int itemcount = 0;
		foreach(HashTable<string,string> ht in itemlist.items())
		{
			Item item = new Item(get_knots(), ht, get_knots().get_browser().has_icon_view());
			item.create();
			itemslist += item;
			add_item(item);
			itemcount += 1;
		}
		scroller.get_container().show_all();
		needs_update = false;
		get_toolbar_button(1).set_sensitive(itemcount > 0);
		get_toolbar_button(2).set_sensitive(itemcount > 1);
		sanitize_buttons();
		gtk_window_set_progress_indicator(get_knots().get_window(), 0);
		if (get_knots().visible_page(get_knots().get_info()))
			get_knots().get_info().check_playlist_status();
		return false;
	}
	
	public void download_items()
	{
		if (download_index == 0 && itemlist != null && itemlist.itemcount() > 0)
		{			
			MessageDialog dialog = new MessageDialog.with_markup(null, 0, Gtk.MessageType.QUESTION, Gtk.ButtonsType.OK_CANCEL, "Transfer all files smaller than 10 MB to device?");
			((Label)dialog.label).selectable = false;
			gtk_window_set_portrait_flags(dialog, Hildon.PortraitFlags.SUPPORT);
			dialog.set_image(new Gtk.Image.from_file("/usr/share/knots/knots_button_playlist.png"));
			var response = dialog.run();
			if (response == ResponseType.OK)
			{
				try
				{
					Thread.create(transfer, true);
				}
				catch (ThreadError e)
				{
				
				}
			}
			dialog.destroy();
		}
		else
			download_index = -1;
	}
	
	public void* transfer()
	{
		download_index += 1;
		string path = Path.build_filename(Environment.get_home_dir(), "MyDocs", "knots", null);
		if (!FileUtils.test(path, FileTest.EXISTS))
			DirUtils.create_with_parents(path, 0755);
		else
		if(!FileUtils.test(path, FileTest.IS_DIR))
		{
			download_index = -1;
			GLib.Idle.add(end_transfer);	
			return null;
		}
		get_knots().get_connection().set_url_cache(false);
		XMList transfer_list = new XMList(get_knots().get_connection().fetch("/external/playlist?id=" + get_selected_playlist_id().to_string()));
		foreach(HashTable<string,string> ht in transfer_list.items())
		{
			string id = (string)ht.lookup("id");
			downloaded_filename = GLib.Filename.display_basename((string)ht.lookup("path"));
			GLib.Idle.add(update_transfer);
			string target = Path.build_filename(path, downloaded_filename, null);
			int filesize = ((string)ht.lookup("size")).to_int();
			bool download = false;
			try
			{
				download = !FileUtils.test(target, FileTest.EXISTS) || File.new_for_path(target).query_info("*", FileQueryInfoFlags.NONE, null).get_size() != filesize;
			}
			catch (GLib.Error e) {}
			if (download)
			{
				string url = (get_knots().get_connection().uses_ssl() ? "https" : "http") + "://" + get_knots().get_connection().address() + "/external/download?id=" + id;
				var session = new Soup.SessionSync ();
				var message = new Soup.Message ("GET", url);
				session.authenticate +=  (sess, msg, auth, retrying) => {
				    if (retrying)
				    {
				    	download_index = -1;
				    	GLib.Idle.add(end_transfer);	
					return;
				    }
				    get_knots().get_connection().authenticate(false);
				    auth.authenticate(get_knots().get_connection().username(),get_knots().get_connection().password());
				};
				message.got_body += (message) => {
					if (message.response_body.length == filesize)
					{
						try
						{
							FileUtils.set_contents(target, (string)message.response_body.data, (int)message.response_body.length);
						}
						catch (GLib.Error e)
						{
							download_index = -1;
							GLib.Idle.add(end_transfer);
							return;
						}
					}
				};
				message.got_chunk += (message) => {
					if (download_index == -1)
						session.abort();
				};
				session.send_message (message);
			}
			if (download_index != -1)
				download_index += 1;
			else
			{
				GLib.Idle.add(end_transfer);
				return null;
			}
		}
		GLib.Idle.add(end_transfer);
		return null;
	}
	
	public bool update_transfer()
	{
		get_knots().get_menu_button("download").set_value(downloaded_filename);
		return false;
	}
	
	public bool end_transfer()
	{
		if (download_index != -1)
			get_knots().alert("Transfer complete");
		else
			get_knots().alert("Transfer aborted");
		download_index = 0;
		get_knots().get_menu_button("download").set_value("");	
		return false;
	}
	
	public void add_item(Item item)
	{	
		var ww = window.get_screen().get_width();
		int item_count = ww / item.width();
		int gap = (ww - item_count * item.width()) / (item_count + 1);
		x += gap;
		((Gtk.Fixed)scroller.get_container()).put(item, x, y);
		x += item.width();
		if (x + item.width() > ww)
		{
			x = 0;
			y += item.height() + 5;
		}
	}
	
	public void re_add_items()
	{
		clear_list();
		foreach(Item item in itemslist)
		{
			if (!item.has_icon_view())
				item.new_width(window.get_screen().get_width());
			add_item(item);
		}
		scroller.get_container().show_all();
	}
	
	public override string[] get_toolbar_items()
	{
		return {"back", "play", "reorder", "new", "delete"};
	}
	
	public void add_to_playlist(int id)
	{
		int pid = get_selected_playlist_id();
		if (pid == -1)
			pid = 1;
		get_knots().get_connection().set_url_cache(false);
		get_knots().get_connection().fetch("/external/add_to_playlist?playlist=" + pid.to_string() + "&id=" + id.to_string());
		if (!get_knots().get_player().is_playing())
			get_knots().alert("Added to playlist");
		else
			get_knots().alert("Added to playlist. Restart playback to reload the playlist.");
		update_list();
		needs_update = true;
	}
	
	public void add_results_to_playlist(string ids)
	{
		int pid = get_selected_playlist_id();
		if (pid == -1)
			pid = 1;
		get_knots().get_connection().set_url_cache(false);
		get_knots().get_connection().fetch("/external/add_results_to_playlist?playlist=" + pid.to_string() + "&ids=" + ids.to_string());
		if (!get_knots().get_player().is_playing())
			get_knots().alert("Added to playlist");
		else
			get_knots().alert("Added to playlist. Restart playback to reload the playlist.");
		update_list();
		needs_update = true;
	}
	
	public int? get_playlist_mediatype()
	{
		if (itemlist.itemcount() > 0)
		{
			HashTable<string,string> ht = itemlist.items()[0];
			return ((string)ht.lookup("mediatype")).to_int();
		}
		return null;
	}
	
	public bool can_add_to_playlist(int mediatype)
	{
		/* don't mix audio and videos */
		foreach(Item item in itemslist)
		{
			int item_type = item.get_item_attribute("mediatype").to_int();
			if ((item_type != 1 && mediatype == 1) || (item_type == 1 && mediatype != 1))
			{
				return false;
			}
		}
		return true;
	}
	
	public void clear_list()
	{
		x = 0;
		y = 0;
		var children = scroller.get_container().get_children();
		foreach (Widget child in children) {
			scroller.get_container().remove(child);
			child = null;
		}
		scroller.scroll_to_top();
	}
	
	public void set_update_need(bool needs_update)
	{
		this.needs_update = needs_update;
	}
	
	public void remove_from_playlist(int id)
	{
		get_knots().get_connection().set_url_cache(false);
		get_knots().get_connection().fetch("/external/remove_from_playlist?pid=" + id.to_string());
		update_items();
	}
	
	public void remove_item_from_playlist(int id)
	{
		get_knots().get_connection().set_url_cache(false);
		get_knots().get_connection().fetch("/external/remove_from_playlist?playlist=" + get_selected_playlist_id().to_string() + "&id=" + id.to_string());
		update_items();
		get_knots().alert("Removed from playlist");
	}
	
	public void share_playlist()
	{
		string[] profile = get_knots().select_transcoding_profile("Select profile for share");
		if (profile[0] != null)
		{
			get_knots().get_connection().set_url_cache(false);
			string url = get_knots().get_connection().fetch("/external/share_playlist?playlist=" + get_selected_playlist_id().to_string() + "&profile=" + profile[0]);
			if (url.str("http") != null)
			{
				Gtk.Clipboard clipboard = Gtk.Clipboard.get(Gdk.Atom.intern_static_string("CLIPBOARD"));
				if (clipboard != null)
				{
					clipboard.set_text(url.strip(), -1);
					get_knots().get_settings().set_value("last_share", url);
					get_knots().alert("Share url copied to clipboard");
				}
				else
				{
					get_knots().alert("Playlist sharing failed");
				}
			}
			else
			{
				get_knots().alert("Playlist sharing failed");
			}
		}
	}
	
	public override void reveal()
	{
		base.reveal();
		get_knots().get_window().queue_draw();
		scroller.enable_clicking(true);
		get_toolbar_button(1).set_sensitive(itemlist.itemcount() > 0);
		get_toolbar_button(2).set_sensitive(itemlist.itemcount() > 1);
		if (this.needs_update)
			update_items();
		else
			sanitize_buttons();
	}
	
	public override string[] show_buttons()
	{
		return {"rotation", "icon_view", "profile"};
	}
	
	public void sanitize_buttons()
	{
		if (itemlist != null && itemlist.itemcount() > 0)
		{
			get_knots().get_menu_button("download").show();
			get_knots().get_menu_button("share").show();
		}
		else
		{
			get_knots().get_menu_button("download").hide();
			get_knots().get_menu_button("share").hide();
		}
	}
}
